#ifndef NPSTAT_ALLSYMBETAPARAMS1D_HH_
#define NPSTAT_ALLSYMBETAPARAMS1D_HH_

/*!
// \file AllSymbetaParams1D.hh
//
// \brief Complete set of parameters for building 1-d filters from the
//        symmetric beta family
//
// Author: I. Volobouev
//
// October 2019
*/

#include <climits>

#include "geners/ClassId.hh"
#include "npstat/stat/BoundaryHandling.hh"

namespace npstat {
    class AllSymbetaParams1D
    {
    public:
        inline AllSymbetaParams1D()
            : bandwidth_(-1.0), degree_(-1.0), binwidth_(-1.0),
              nbins_(0), excludedBin_(0), symbetaPower_(0),
              excludeCentralPoint_(false) {}

        AllSymbetaParams1D(
            int symbetaPower, double bandwidth, double degree,
            unsigned nbins, double binwidth, const BoundaryHandling& bh,
            unsigned excludedBin = UINT_MAX, bool excludeCentralPoint = false);

        bool operator<(const AllSymbetaParams1D& r) const;

        inline bool operator>(const AllSymbetaParams1D& r) const
            {return r < *this;}

        bool operator==(const AllSymbetaParams1D& r) const;

        inline bool operator!=(const AllSymbetaParams1D& r) const
            {return !(*this == r);}

        // I/O methods needed for writing
        inline gs::ClassId classId() const {return gs::ClassId(*this);}
        bool write(std::ostream& of) const;

        // I/O methods needed for reading
        static inline const char* classname()
            {return "npstat::Private::AllSymbetaParams1D";}
        static inline unsigned version() {return 1;}
        static void restore(const gs::ClassId& id, std::istream& in,
                            AllSymbetaParams1D* ptr);

        inline double get_bandwidth() const {return bandwidth_;}
        inline double get_degree() const {return degree_;}
        inline double get_binwidth() const {return binwidth_;}
        inline unsigned get_nbins() const {return nbins_;}
        inline unsigned get_excludedBin() const {return excludedBin_;}
        inline int get_symbetaPower() const {return symbetaPower_;}
        inline const BoundaryHandling& get_bm() const {return bm_;}
        inline bool get_excludeCentralPoint() const 
            {return excludeCentralPoint_;}

    private:
        double bandwidth_;
        double degree_;
        double binwidth_;
        unsigned nbins_;
        unsigned excludedBin_;
        int symbetaPower_;
        BoundaryHandling bm_;
        bool excludeCentralPoint_;
    };
 }

#endif // NPSTAT_ALLSYMBETAPARAMS1D_HH_
