#include "geners/binaryIO.hh"
#include "geners/IOException.hh"

#include "npstat/stat/GaussianMixtureEntry.hh"

namespace npstat {
    bool GaussianMixtureEntry::write(std::ostream& os) const
    {
        gs::write_pod(os, w_);
        gs::write_pod(os, mean_);
        gs::write_pod(os, stdev_);
        return !os.fail();
    }

    bool GaussianMixtureEntry::operator==(const GaussianMixtureEntry& r) const
    {
        return w_ == r.w_ && mean_ == r.mean_ && stdev_ == r.stdev_;
    }

    void GaussianMixtureEntry::restore(const gs::ClassId& id, std::istream& in,
                                       GaussianMixtureEntry* entry)
    {
        static const gs::ClassId current(
            gs::ClassId::makeId<GaussianMixtureEntry>());
        current.ensureSameId(id);

        assert(entry);
        gs::read_pod(in, &entry->w_);
        gs::read_pod(in, &entry->mean_);
        gs::read_pod(in, &entry->stdev_);
        if (in.fail()) throw gs::IOReadFailure(
            "In npstat::GaussianMixtureEntry::restore: input stream failure");
    }
}
