#include <cassert>
#include <cstring>
#include <sstream>
#include <stdexcept>

#include "npstat/stat/LikelihoodStatisticType.hh"

static const char* names[] = {
    "W",
    "W2",
    "W3",
    "S",
    "LR"
};

static const unsigned nNames = sizeof(names)/sizeof(names[0]);

namespace npstat {
    LikelihoodStatisticType parseLikelihoodStatisticType(const char* cName)
    {
        for (unsigned i=0; i<nNames; ++i)
            if (strcmp(names[i], cName) == 0)
                return static_cast<LikelihoodStatisticType>(i);
        std::ostringstream os;
        os << "In npstat::parseLikelihoodStatisticType: invalid argument \""
           << cName << "\". Must be one of "
           << validLikelihoodStatisticNames() << '.';
        throw std::invalid_argument(os.str());
    }

    const char* likelihoodStatisticName(const LikelihoodStatisticType t)
    {
        const unsigned i = static_cast<unsigned>(t);
        assert(i < nNames);
        return names[i];
    }

    std::string validLikelihoodStatisticNames()
    {
        std::ostringstream os;
        for (unsigned i=0; i<nNames; ++i)
        {
            if (i == 0)
                os << names[i];
            else if (i == nNames - 1)
                os << " or " << names[i];
            else
                os << ", " << names[i];
        }
        return os.str();
    }
}
