#ifndef NPSTAT_CUMULANTUNCERTAINTIES_HH_
#define NPSTAT_CUMULANTUNCERTAINTIES_HH_

/*!
// \file cumulantUncertainties.hh
//
// \brief Uncertainties of sample cumulants using population cumulants
//
// Author: I. Volobouev
//
// July 2019
*/

namespace npstat {
    /**
    // Array "cumulants" should have at least maxCumOrder+1 elements.
    // Array "uncertainties" should have at least maxUncertOrder+1 elements.
    // maxCumOrder must be larger than or equal to 2*maxUncertOrder.
    // On exit, uncertainties[1] will contain the expected uncertainty
    // of the first cumulant (i.e., the mean) for the sample of size
    // "sampleSize", uncertainties[2] will contain the expected uncertainty
    // of the second cumulant, etc.
    //
    // The formulae used can be found in Chapter 12 of Stuart and Ord.
    */
    template<typename Real>
    void cumulantUncertainties(const Real* cumulants, unsigned maxCumOrder,
                               unsigned long sampleSize,
                               Real* uncertainties, unsigned maxUncertOrder);
}

#include "npstat/stat/cumulantUncertainties.icc"

#endif // NPSTAT_CUMULANTUNCERTAINTIES_HH_
