#ifndef NPSTAT_SCANDENSITYND_HH_
#define NPSTAT_SCANDENSITYND_HH_

#include <cassert>
#include <stdexcept>

#include "npstat/stat/AbsDistributionND.hh"
#include "npstat/nm/BoxNDScanner.hh"
#include "npstat/wrap/NumpyTypecode.hh"

namespace npstat {
    inline PyObject* scanDensityND(
        const AbsDistributionND& distro,
        const BoxND<double>& boundingBox,
        const std::vector<unsigned>& shape)
    {
        const unsigned dim = distro.dim();
        if (!dim || dim != boundingBox.dim() || dim != shape.size())
            throw std::invalid_argument(
                "In npstat::scanDensityND: "
                "incompatible argument dimensionalities");
        assert(dim <= CHAR_BIT*sizeof(unsigned long));

        npy_intp sh[CHAR_BIT*sizeof(unsigned long)];
        for (unsigned i=0; i<dim; ++i)
            sh[i] = shape[i];
        const int typenum = NumpyTypecode<double>::code;
        PyObject* arr  = PyArray_SimpleNew(dim, &sh[0], typenum);
        if (arr)
        {
            double* data = (double*)PyArray_DATA((PyArrayObject*)arr);
            double coords[CHAR_BIT*sizeof(unsigned long)];
            for (BoxNDScanner<double> scan(boundingBox, shape);
                 scan.isValid(); ++scan)
            {
                scan.getCoords(&coords[0], dim);
                data[scan.state()] = distro.density(&coords[0], dim);
            }
        }
        return arr;
    }
}

#endif // NPSTAT_SCANDENSITYND_HH_
